@@ -1,8 +1,7 @@
 """The ``celery purge`` program, used to delete messages from queues."""
 import click
 
-from celery.bin.base import (COMMA_SEPARATED_LIST, CeleryCommand,
-                             CeleryOption, handle_preload_options)
+from celery.bin.base import COMMA_SEPARATED_LIST, CeleryCommand, CeleryOption, handle_preload_options
 from celery.utils import text
 
 
