@@ -1,26 +1,31 @@
-# -*- coding: utf-8 -*-
 """Utilities related to dates, times, intervals, and timezones."""
-from __future__ import absolute_import, print_function, unicode_literals
+from __future__ import annotations
 
 import numbers
 import os
 import random
+import sys
 import time as _time
 from calendar import monthrange
-from datetime import date, datetime, timedelta, tzinfo
+from datetime import date, datetime, timedelta
+from datetime import timezone as datetime_timezone
+from datetime import tzinfo
+from types import ModuleType
+from typing import Any, Callable
 
+from dateutil import tz as dateutil_tz
 from kombu.utils.functional import reprcall
 from kombu.utils.objects import cached_property
-from pytz import AmbiguousTimeError, FixedOffset
-from pytz import timezone as _timezone
-from pytz import utc
-
-from celery.five import PY3, python_2_unicode_compatible, string_t
 
 from .functional import dictfilter
-from .iso8601 import parse_iso8601
 from .text import pluralize
 
+if sys.version_info >= (3, 9):
+    from zoneinfo import ZoneInfo
+else:
+    from backports.zoneinfo import ZoneInfo
+
+
 __all__ = (
     'LocalTimezone', 'timezone', 'maybe_timedelta',
     'delta_resolution', 'remaining', 'rate', 'weekday',
@@ -53,17 +58,17 @@ ZERO = timedelta(0)
 _local_timezone = None
 
 
-@python_2_unicode_compatible
 class LocalTimezone(tzinfo):
-    """Local time implementation.
+    """Local time implementation. Provided in _Zone to the app when `enable_utc` is disabled.
+    Otherwise, _Zone provides a UTC ZoneInfo instance as the timezone implementation for the application.
 
     Note:
         Used only when the :setting:`enable_utc` setting is disabled.
     """
 
-    _offset_cache = {}
+    _offset_cache: dict[int, tzinfo] = {}
 
-    def __init__(self):
+    def __init__(self) -> None:
         # This code is moved in __init__ to execute it as late as possible
         # See get_default_timezone().
         self.STDOFFSET = timedelta(seconds=-_time.timezone)
@@ -72,38 +77,32 @@ class LocalTimezone(tzinfo):
         else:
             self.DSTOFFSET = self.STDOFFSET
         self.DSTDIFF = self.DSTOFFSET - self.STDOFFSET
-        tzinfo.__init__(self)
+        super().__init__()
 
-    def __repr__(self):
-        return '<LocalTimezone: UTC{0:+03d}>'.format(
-            int(self.DSTOFFSET.total_seconds() / 3600),
-        )
+    def __repr__(self) -> str:
+        return f'<LocalTimezone: UTC{int(self.DSTOFFSET.total_seconds() / 3600):+03d}>'
 
-    def utcoffset(self, dt):
+    def utcoffset(self, dt: datetime) -> timedelta:
         return self.DSTOFFSET if self._isdst(dt) else self.STDOFFSET
 
-    def dst(self, dt):
+    def dst(self, dt: datetime) -> timedelta:
         return self.DSTDIFF if self._isdst(dt) else ZERO
 
-    def tzname(self, dt):
+    def tzname(self, dt: datetime) -> str:
         return _time.tzname[self._isdst(dt)]
 
-    if PY3:  # pragma: no cover
+    def fromutc(self, dt: datetime) -> datetime:
+        # The base tzinfo class no longer implements a DST
+        # offset aware .fromutc() in Python 3 (Issue #2306).
+        offset = int(self.utcoffset(dt).seconds / 60.0)
+        try:
+            tz = self._offset_cache[offset]
+        except KeyError:
+            tz = self._offset_cache[offset] = datetime_timezone(
+                timedelta(minutes=offset))
+        return tz.fromutc(dt.replace(tzinfo=tz))
 
-        def fromutc(self, dt):
-            # The base tzinfo class no longer implements a DST
-            # offset aware .fromutc() in Python 3 (Issue #2306).
-
-            # I'd rather rely on pytz to do this, than port
-            # the C code from cpython's fromutc [asksol]
-            offset = int(self.utcoffset(dt).seconds / 60.0)
-            try:
-                tz = self._offset_cache[offset]
-            except KeyError:
-                tz = self._offset_cache[offset] = FixedOffset(offset)
-            return tz.fromutc(dt.replace(tzinfo=tz))
-
-    def _isdst(self, dt):
+    def _isdst(self, dt: datetime) -> bool:
         tt = (dt.year, dt.month, dt.day,
               dt.hour, dt.minute, dt.second,
               dt.weekday(), 0, 0)
@@ -112,61 +111,70 @@ class LocalTimezone(tzinfo):
         return tt.tm_isdst > 0
 
 
-class _Zone(object):
+class _Zone:
+    """Timezone class that provides the timezone for the application.
+    If `enable_utc` is disabled, LocalTimezone is provided as the timezone provider through local().
+    Otherwise, this class provides a UTC ZoneInfo instance as the timezone provider for the application.
+
+    Additionally this class provides a few utility methods for converting datetimes.
+    """
+
+    def tz_or_local(self, tzinfo: tzinfo | None = None) -> tzinfo:
+        """Return either our local timezone or the provided timezone."""
 
-    def tz_or_local(self, tzinfo=None):
         # pylint: disable=redefined-outer-name
         if tzinfo is None:
             return self.local
         return self.get_timezone(tzinfo)
 
-    def to_local(self, dt, local=None, orig=None):
+    def to_local(self, dt: datetime, local=None, orig=None):
+        """Converts a datetime to the local timezone."""
+
         if is_naive(dt):
             dt = make_aware(dt, orig or self.utc)
         return localize(dt, self.tz_or_local(local))
 
-    if PY3:  # pragma: no cover
+    def to_system(self, dt: datetime) -> datetime:
+        """Converts a datetime to the system timezone."""
 
-        def to_system(self, dt):
-            # tz=None is a special case since Python 3.3, and will
-            # convert to the current local timezone (Issue #2306).
-            return dt.astimezone(tz=None)
+        # tz=None is a special case since Python 3.3, and will
+        # convert to the current local timezone (Issue #2306).
+        return dt.astimezone(tz=None)
 
-    else:
-
-        def to_system(self, dt):  # noqa
-            return localize(dt, self.local)
-
-    def to_local_fallback(self, dt):
+    def to_local_fallback(self, dt: datetime) -> datetime:
+        """Converts a datetime to the local timezone, or the system timezone."""
         if is_naive(dt):
             return make_aware(dt, self.local)
         return localize(dt, self.local)
 
-    def get_timezone(self, zone):
-        if isinstance(zone, string_t):
-            return _timezone(zone)
+    def get_timezone(self, zone: str | tzinfo) -> tzinfo:
+        """Returns ZoneInfo timezone if the provided zone is a string, otherwise return the zone."""
+        if isinstance(zone, str):
+            return ZoneInfo(zone)
         return zone
 
     @cached_property
-    def local(self):
+    def local(self) -> LocalTimezone:
+        """Return LocalTimezone instance for the application."""
         return LocalTimezone()
 
     @cached_property
-    def utc(self):
+    def utc(self) -> tzinfo:
+        """Return UTC timezone created with ZoneInfo."""
         return self.get_timezone('UTC')
 
 
 timezone = _Zone()
 
 
-def maybe_timedelta(delta):
+def maybe_timedelta(delta: int) -> timedelta:
     """Convert integer to timedelta, if argument is an integer."""
     if isinstance(delta, numbers.Real):
         return timedelta(seconds=delta)
     return delta
 
 
-def delta_resolution(dt, delta):
+def delta_resolution(dt: datetime, delta: timedelta) -> datetime:
     """Round a :class:`~datetime.datetime` to the resolution of timedelta.
 
     If the :class:`~datetime.timedelta` is in days, the
@@ -189,7 +197,9 @@ def delta_resolution(dt, delta):
     return dt
 
 
-def remaining(start, ends_in, now=None, relative=False):
+def remaining(
+        start: datetime, ends_in: timedelta, now: Callable | None = None,
+        relative: bool = False) -> timedelta:
     """Calculate the remaining time for a start date and a timedelta.
 
     For example, "how many seconds left for 30 seconds after start?"
@@ -207,7 +217,9 @@ def remaining(start, ends_in, now=None, relative=False):
         ~datetime.timedelta: Remaining time.
     """
     now = now or datetime.utcnow()
-    if str(start.tzinfo) == str(now.tzinfo) and now.utcoffset() != start.utcoffset():
+    if str(
+            start.tzinfo) == str(
+            now.tzinfo) and now.utcoffset() != start.utcoffset():
         # DST started/ended
         start = start.replace(tzinfo=now.tzinfo)
     end_date = start + ends_in
@@ -215,22 +227,22 @@ def remaining(start, ends_in, now=None, relative=False):
         end_date = delta_resolution(end_date, ends_in).replace(microsecond=0)
     ret = end_date - now
     if C_REMDEBUG:  # pragma: no cover
-        print('rem: NOW:%r START:%r ENDS_IN:%r END_DATE:%s REM:%s' % (
+        print('rem: NOW:{!r} START:{!r} ENDS_IN:{!r} END_DATE:{} REM:{}'.format(
             now, start, ends_in, end_date, ret))
     return ret
 
 
-def rate(r):
+def rate(r: str) -> float:
     """Convert rate string (`"100/m"`, `"2/h"` or `"0.5/s"`) to seconds."""
     if r:
-        if isinstance(r, string_t):
+        if isinstance(r, str):
             ops, _, modifier = r.partition('/')
             return RATE_MODIFIER_MAP[modifier or 's'](float(ops)) or 0
         return r or 0
     return 0
 
 
-def weekday(name):
+def weekday(name: str) -> int:
     """Return the position of a weekday: 0 - 7, where 0 is Sunday.
 
     Example:
@@ -245,7 +257,9 @@ def weekday(name):
         raise KeyError(name)
 
 
-def humanize_seconds(secs, prefix='', sep='', now='now', microseconds=False):
+def humanize_seconds(
+        secs: int, prefix: str = '', sep: str = '', now: str = 'now',
+        microseconds: bool = False) -> str:
     """Show seconds in human form.
 
     For example, 60 becomes "1 minute", and 7200 becomes "2 hours".
@@ -260,85 +274,91 @@ def humanize_seconds(secs, prefix='', sep='', now='now', microseconds=False):
     for unit, divider, formatter in TIME_UNITS:
         if secs >= divider:
             w = secs / float(divider)
-            return '{0}{1}{2} {3}'.format(prefix, sep, formatter(w),
-                                          pluralize(w, unit))
+            return '{}{}{} {}'.format(prefix, sep, formatter(w),
+                                      pluralize(w, unit))
     if microseconds and secs > 0.0:
         return '{prefix}{sep}{0:.2f} seconds'.format(
             secs, sep=sep, prefix=prefix)
     return now
 
 
-def maybe_iso8601(dt):
+def maybe_iso8601(dt: datetime | str | None) -> None | datetime:
     """Either ``datetime | str -> datetime`` or ``None -> None``."""
     if not dt:
         return
     if isinstance(dt, datetime):
         return dt
-    return parse_iso8601(dt)
+    return datetime.fromisoformat(dt)
 
 
-def is_naive(dt):
-    """Return :const:`True` if :class:`~datetime.datetime` is naive."""
+def is_naive(dt: datetime) -> bool:
+    """Return True if :class:`~datetime.datetime` is naive, meaning it doesn't have timezone info set."""
     return dt.tzinfo is None or dt.tzinfo.utcoffset(dt) is None
 
 
-def make_aware(dt, tz):
+def _can_detect_ambiguous(tz: tzinfo) -> bool:
+    """Helper function to determine if a timezone can detect ambiguous times using dateutil."""
+
+    return isinstance(tz, ZoneInfo) or hasattr(tz, "is_ambiguous")
+
+
+def _is_ambigious(dt: datetime, tz: tzinfo) -> bool:
+    """Helper function to determine if a timezone is ambiguous using python's dateutil module.
+
+    Returns False if the timezone cannot detect ambiguity, or if there is no ambiguity, otherwise True.
+
+    In order to detect ambiguous datetimes, the timezone must be built using ZoneInfo, or have an is_ambiguous
+    method. Previously, pytz timezones would throw an AmbiguousTimeError if the localized dt was ambiguous,
+    but now we need to specifically check for ambiguity with dateutil, as pytz is deprecated.
+    """
+
+    return _can_detect_ambiguous(tz) and dateutil_tz.datetime_ambiguous(dt)
+
+
+def make_aware(dt: datetime, tz: tzinfo) -> datetime:
     """Set timezone for a :class:`~datetime.datetime` object."""
-    try:
-        _localize = tz.localize
-    except AttributeError:
-        return dt.replace(tzinfo=tz)
-    else:
-        # works on pytz timezones
-        try:
-            return _localize(dt, is_dst=None)
-        except AmbiguousTimeError:
-            return min(_localize(dt, is_dst=True),
-                       _localize(dt, is_dst=False))
+
+    dt = dt.replace(tzinfo=tz)
+    if _is_ambigious(dt, tz):
+        dt = min(dt.replace(fold=0), dt.replace(fold=1))
+    return dt
 
 
-def localize(dt, tz):
-    """Convert aware :class:`~datetime.datetime` to another timezone."""
+def localize(dt: datetime, tz: tzinfo) -> datetime:
+    """Convert aware :class:`~datetime.datetime` to another timezone.
+
+    Using a ZoneInfo timezone will give the most flexibility in terms of ambiguous DST handling.
+    """
     if is_naive(dt):  # Ensure timezone aware datetime
         dt = make_aware(dt, tz)
-    if dt.tzinfo == utc:
+    if dt.tzinfo == ZoneInfo("UTC"):
         dt = dt.astimezone(tz)  # Always safe to call astimezone on utc zones
-    try:
-        _normalize = tz.normalize
-    except AttributeError:  # non-pytz tz
-        return dt
-    else:
-        try:
-            return _normalize(dt, is_dst=None)
-        except TypeError:
-            return _normalize(dt)
-        except AmbiguousTimeError:
-            return min(_normalize(dt, is_dst=True),
-                       _normalize(dt, is_dst=False))
+    return dt
 
 
-def to_utc(dt):
+def to_utc(dt: datetime) -> datetime:
     """Convert naive :class:`~datetime.datetime` to UTC."""
     return make_aware(dt, timezone.utc)
 
 
-def maybe_make_aware(dt, tz=None):
+def maybe_make_aware(dt: datetime, tz: tzinfo | None = None,
+                     naive_as_utc: bool = True) -> datetime:
     """Convert dt to aware datetime, do nothing if dt is already aware."""
     if is_naive(dt):
-        dt = to_utc(dt)
+        if naive_as_utc:
+            dt = to_utc(dt)
         return localize(
             dt, timezone.utc if tz is None else timezone.tz_or_local(tz),
         )
     return dt
 
 
-@python_2_unicode_compatible
-class ffwd(object):
+class ffwd:
     """Version of ``dateutil.relativedelta`` that only supports addition."""
 
     def __init__(self, year=None, month=None, weeks=0, weekday=None, day=None,
                  hour=None, minute=None, second=None, microsecond=None,
-                 **kwargs):
+                 **kwargs: Any):
         # pylint: disable=redefined-outer-name
         # weekday is also a function in outer scope.
         self.year = year
@@ -353,11 +373,11 @@ class ffwd(object):
         self.days = weeks * 7
         self._has_time = self.hour is not None or self.minute is not None
 
-    def __repr__(self):
+    def __repr__(self) -> str:
         return reprcall('ffwd', (), self._fields(weeks=self.weeks,
                                                  weekday=self.weekday))
 
-    def __radd__(self, other):
+    def __radd__(self, other: Any) -> timedelta:
         if not isinstance(other, date):
             return NotImplemented
         year = self.year or other.year
@@ -369,7 +389,7 @@ class ffwd(object):
             ret += timedelta(days=(7 - ret.weekday() + self.weekday) % 7)
         return ret + timedelta(days=self.days)
 
-    def _fields(self, **extra):
+    def _fields(self, **extra: Any) -> dict[str, Any]:
         return dictfilter({
             'year': self.year, 'month': self.month, 'day': self.day,
             'hour': self.hour, 'minute': self.minute,
@@ -377,29 +397,32 @@ class ffwd(object):
         }, **extra)
 
 
-def utcoffset(time=_time, localtime=_time.localtime):
+def utcoffset(
+        time: ModuleType = _time,
+        localtime: Callable[..., _time.struct_time] = _time.localtime) -> float:
     """Return the current offset to UTC in hours."""
     if localtime().tm_isdst:
         return time.altzone // 3600
     return time.timezone // 3600
 
 
-def adjust_timestamp(ts, offset, here=utcoffset):
+def adjust_timestamp(ts: float, offset: int,
+                     here: Callable[..., float] = utcoffset) -> float:
     """Adjust timestamp based on provided utcoffset."""
     return ts - (offset - here()) * 3600
 
 
 def get_exponential_backoff_interval(
-    factor,
-    retries,
-    maximum,
-    full_jitter=False
-):
+    factor: int,
+    retries: int,
+    maximum: int,
+    full_jitter: bool = False
+) -> int:
     """Calculate the exponential backoff wait time."""
     # Will be zero if factor equals 0
     countdown = min(maximum, factor * (2 ** retries))
     # Full jitter according to
-    # https://www.awsarchitectureblog.com/2015/03/backoff.html
+    # https://aws.amazon.com/blogs/architecture/exponential-backoff-and-jitter/
     if full_jitter:
         countdown = random.randrange(countdown + 1)
     # Adjust according to maximum wait time and account for negative values.
